import dataset from "assets/dataset_small.json"
import {
  collection,
  getDocs,
  limit,
  query,
  where,
  type DocumentData,
} from "firebase/firestore/lite"

import type { PlasmoMessaging } from "@plasmohq/messaging"

// import initialized db instance
import { db } from "~background"
import type { data_small } from "~interfaces/data_small"

interface url_data {
  name: string
  url: string
}

const data: data_small[] = dataset

async function queryUrls(currentTab: chrome.tabs.Tab) {
  let answer
  const url = currentTab.url.split("/")[2]
  const urlsCollection = query(
    collection(db, "urls"),
    where("url", "==", url),
    limit(1),
  )
  const brandsSnapshot = await getDocs(urlsCollection)
  if (brandsSnapshot.empty) {
    return answer
  } else {
    brandsSnapshot.forEach((doc) => {
      answer = doc.data()
      console.log(answer)
    })
    return answer
  }
}

function ifExists(tokens: string) {
  const answer = data.find((brand) =>
    tokens.toLowerCase().includes(brand.brand.toLowerCase()),
  )
  return answer
}

function getAnswers(tokens: string, validUrl: DocumentData) {
  let secondAnswer
  let mainAnswer: data_small = ifExists(tokens)
  // chack if mainAnswer isnt undefined and identify any brands from our dataset in the url
  if (
    mainAnswer &&
    validUrl.name.toLowerCase().includes(mainAnswer.brand.toLowerCase())
  ) {
    secondAnswer = mainAnswer
    mainAnswer = ifExists(tokens.replace(secondAnswer.brand, ""))
  } else {
    secondAnswer = ifExists(validUrl.name)
  }
  //make sure that if only second answer is found then it is displayed
  //ensure that if main and second answers are the same then only display it once
  return { mainAnswer, secondAnswer }
}

const handler: PlasmoMessaging.MessageHandler = async (req, res) => {
  let answer
  const queryOptions = { active: true, lastFocusedWindow: true }
  const [tab]: chrome.tabs.Tab[] = await chrome.tabs.query(queryOptions)
  const validUrl = await queryUrls(tab)
  const tokens: string = await tab.title
  if (validUrl) {
    answer = getAnswers(tokens, validUrl)
  }
  if (!answer && !validUrl) {
    res.send(undefined)
  } else {
    res.send({
      answer,
      validUrl,
    })
  }
}

export default handler
